package.path = package.path .. ";data/scripts/lib/?.lua"
package.path = package.path .. ";data/scripts/?.lua"

require ("utility")

local iconColor = ColorRGB(0.5, 0.5, 0.5)


local function fillWeaponTooltipData(obj, tooltip)

    -- primary stats, one by one
    local fontSize = 14
    local lineHeight = 20

    if obj.damage > 0 then
        if obj.continuousBeam == 1 then

            local line = TooltipLine(lineHeight, fontSize)
            line.ltext = "Damage /s"
            line.rtext = round(obj.dps, 1)
            line.icon = "data/textures/icons/screen-impact.png";
            line.iconColor = iconColor
            tooltip:addLine(line)

        else
            -- damage
            local line = TooltipLine(lineHeight, fontSize)
            line.ltext = "Damage"
            line.rtext = round(obj.damage, 1)
            if obj.shotsPerFiring > 1 then
                line.rtext = line.rtext .. " x" .. obj.shotsPerFiring
            end
            line.icon = "data/textures/icons/screen-impact.png";
            line.iconColor = iconColor
            tooltip:addLine(line)

            -- fire rate
            local line = TooltipLine(lineHeight, fontSize)
            line.ltext = "Fire Rate"
            line.rtext = round(obj.fireRate, 1)
            line.icon = "data/textures/icons/bullets.png";
            line.iconColor = iconColor
            tooltip:addLine(line)
        end
    end

    if obj.otherForce > 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Push"
        line.rtext = round(obj.otherForce / 1000, 1) .. " kN"
        line.icon = "data/textures/icons/back-forth.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    elseif obj.otherForce < 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Pull"
        line.rtext = round(-obj.otherForce / 1000, 1) .. " kN"
        line.icon = "data/textures/icons/back-forth.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    if obj.selfForce > 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Self Push"
        line.rtext = round(obj.selfForce / 1000, 1) .. " kN"
        line.icon = "data/textures/icons/back-forth.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    elseif obj.selfForce < 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Self Pull"
        line.rtext = round(-obj.selfForce / 1000, 1) .. " kN"
        line.icon = "data/textures/icons/back-forth.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    if obj.efficiency > 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Efficiency"
        line.rtext = round(obj.efficiency * 100)
        line.icon = "data/textures/icons/recycle.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    if obj.hullRepairRate > 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Hull Dmg /s"
        line.rtext = round(obj.hullRepairRate, 1)
        line.icon = "data/textures/icons/health-normal.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    if obj.shieldRepairRate > 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Shield Dmg /s"
        line.rtext = round(obj.shieldRepairRate, 1)
        line.icon = "data/textures/icons/health-normal.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Range"
    line.rtext = round(obj.reach * 10 / 1000, 2)
    line.icon = "data/textures/icons/target-shot.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    if obj.coolingType == 1 or obj.coolingType == 2 then

        local line = TooltipLine(lineHeight, fontSize)

        if obj.coolingType == 2 then
            line.ltext = "Energy /s"
        else
            line.ltext = "Energy /shot"
        end
        line.rtext = round(obj.baseEnergyPerSecond)
        line.icon = "data/textures/icons/electric.png";
        line.iconColor = iconColor
        tooltip:addLine(line)

        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Energy Increase /s"
        line.rtext = round(obj.energyIncreasePerSecond, 1)
        line.icon = "data/textures/icons/electric.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    tooltip:addLine(TooltipLine(15, 15))

    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Material"
    line.rtext = obj.material.name
    line.rcolor = obj.material.color
    line.icon = "data/textures/icons/metal-bar.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

end

local function fillDescriptions(obj, tooltip)

    -- now count the lines, as there will have to be lines inserted
    -- to make sure that the icon of the weapon won't overlap with the stats
    local extraLines = 0
    local fontSize = 14
    local lineHeight = 18

    -- one line for flavor text
    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = obj.flavorText
    line.lcolor = ColorRGB(1.0, 0.7, 0.7)
    line.lbold = 1
    tooltip:addLine(line)

    extraLines = extraLines + 1

    local descriptions = {obj:getDescriptions()}

    for _, desc in pairs(descriptions) do
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = desc
        tooltip:addLine(line)

        extraLines = extraLines + 1
    end

    for i = 1, 3 - extraLines do
        -- empty line
        tooltip:addLine(TooltipLine(15, 15))
    end

end

function makeTurretTooltip(turret)
    local tooltip = Tooltip()

        -- create tool tip
    tooltip.icon = turret.weaponIcon

    local title = turret.weaponPrefix .. " Turret";

    -- add number predicate
    local prefix = "";

    if turret.numVisibleWeapons == 1 then
        prefix = ""
    elseif turret.numVisibleWeapons == 2 then
        prefix = "Double "
    elseif turret.numVisibleWeapons == 3 then
        prefix = "Triple "
    elseif turret.numVisibleWeapons == 4 then
        prefix = "Quad "
    else
        prefix = "Multi "
    end

    if turret.efficiency > 0 then
        prefix = prefix .. turret.material.name .. " "
    end

    if prefix ~= "" then
        title = prefix .. title;
    end

    -- head line
    local line = TooltipLine(30, 15)
    line.ctext = title
    line.ccolor = turret.rarity.color
    line.cbold = 1
    tooltip:addLine(line)

    local fontSize = 14;
    local lineHeight = 20;

    fillWeaponTooltipData(turret, tooltip)

    -- size
    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Size"
    line.rtext = round(turret.size, 1)
    line.icon = "data/textures/icons/shotgun.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    -- crew requirements
    local crew = turret:getCrew()

    for i = 0, CrewProfessionType.Number - 1 do

        local amount = crew:getMembers(i)

        if amount > 0 then
            local profession = CrewProfession(i)

            local line = TooltipLine(lineHeight, fontSize)
            line.ltext = profession.name
            line.rtext = round(amount)
            line.icon = profession.icon;
            line.iconColor = iconColor
            tooltip:addLine(line)

        end
    end

    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    fillDescriptions(turret, tooltip)

    return tooltip
end



function makeFighterTooltip(fighter)

    -- create tool tip
    local tooltip = Tooltip()
    tooltip.icon = fighter.weaponIcon

    -- title
    local title = fighter.weaponPrefix .. " Fighter"

    local line = TooltipLine(30, 15)
    line.ctext = title
    line.ccolor = fighter.rarity.color
    line.cbold = 1
    tooltip:addLine(line)

    -- primary stats, one by one
    local fontSize = 14
    local lineHeight = 20

    fillWeaponTooltipData(fighter, tooltip)

    -- size
    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Size"
    line.rtext = round(fighter.volume)
    line.icon = "data/textures/icons/fighter.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    -- durability
    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Durability"
    line.rtext = round(fighter.durability)
    line.icon = "data/textures/icons/health-normal.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

    if fighter.shield > 0 then
        local line = TooltipLine(lineHeight, fontSize)
        line.ltext = "Shield"
        line.rtext = round(fighter.durability)
        line.icon = "data/textures/icons/health-normal.png";
        line.iconColor = iconColor
        tooltip:addLine(line)
    end

    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    -- maneuverability
    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Maneuverability"
    line.rtext = round(fighter.turningSpeed, 2)
    line.icon = "data/textures/icons/dodge.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

    -- velocity
    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = "Speed"
    line.rtext = round(fighter.maxVelocity * 10.0)
    line.icon = "data/textures/icons/afterburn.png";
    line.iconColor = iconColor
    tooltip:addLine(line)

    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    -- crew requirements
    local pilot = CrewProfession(CrewProfessionType.Pilot)

    local line = TooltipLine(lineHeight, fontSize)
    line.ltext = pilot.name
    line.rtext = round(fighter.crew)
    line.icon = pilot.icon
    line.iconColor = iconColor
    tooltip:addLine(line)


    -- empty line
    tooltip:addLine(TooltipLine(15, 15))

    fillDescriptions(fighter, tooltip)

    return tooltip
end
