package.path = package.path .. ";data/scripts/lib/?.lua"

function lerp (factor, lowerBound, upperBound, lowerValue, upperValue)
    if lowerBound > upperBound then
        lowerBound, upperBound = upperBound, lowerBound
        lowerValue, upperValue = upperValue, lowerValue
    end

    if lowerBound == upperBound then
        return lowerValue
    end

    local value = math.min(1.0, math.max(0.0, (factor - lowerBound) / (upperBound - lowerBound)))

    return lowerValue + (upperValue - lowerValue) * value
end

function round(num, idp)
    local mult = 10^(idp or 0)
    if num >= 0 then return math.floor(num * mult + 0.5) / mult
    else return math.ceil(num * mult - 0.5) / mult end
end

function getRandomEntry(tbl)
    return tbl[getInt(1, tablelength(tbl))]
end

function getDistribution(numElements, variation)
    assert(variation >= 0 and variation < 1, "Variation must be between [0 , 1)")
    assert(numElements > 0, "numElements must be > 0")

    local result = {}

    variation = variation * 0.5

    local sum = 0
    for i = 0, numElements - 1, 1 do
        local value = getFloat(0.5 - variation, 0.5 + variation)

        sum = sum + value

        result[i] = value
    end

    for i = 0, numElements - 1, 1 do
        result[i] = result[i] / sum
    end

    return result

end

function getValueFromDistribution(distribution)

    local thresholds = {}

    local sum = 0.0

    for key, value in pairs(distribution) do

        local t = {}

        t.lower = sum
        sum = sum + value
        t.upper = sum

        thresholds[key] = t
    end

    local rnd = math.random() * sum
    local lastkey
    for key, value in pairs(thresholds) do

        if rnd >= value.lower and rnd < value.upper then
            return key
        end

        lastkey = key
    end

    return lastkey
end

function createReadableTimeString(seconds)

    seconds = math.floor(seconds)

    local hours = math.floor(seconds / 3600)
    seconds = seconds - hours * 3600

    local minutes = math.floor(seconds / 60)
    seconds = seconds - minutes * 60

    local result = ""

    if hours > 0 then
        return hours .. " hours, " .. minutes .. " minutes"
    end

    if minutes > 0 then
        return minutes .. " minutes, " .. seconds .. " seconds"
    end

    return  seconds .. " seconds"

end

function toRomanLiterals(number)

    local result = ""
    if number < 0 then
        number = -number
        result = "-"
    end

    while number >= 1000 do
        number = number - 1000
        result = result .. "M"
    end

    if number >= 900 then result = result .. "CM"; number = number - 900
    elseif number >= 800 then result = result .. "DCCC"; number = number - 800
    elseif number >= 700 then result = result .. "DCC"; number = number - 700
    elseif number >= 600 then result = result .. "DC"; number = number - 600
    elseif number >= 500 then result = result .. "D"; number = number - 500
    elseif number >= 400 then result = result .. "CD"; number = number - 400
    elseif number >= 300 then result = result .. "CCC"; number = number - 300
    elseif number >= 200 then result = result .. "CC"; number = number - 200
    elseif number >= 100 then result = result .. "C"; number = number - 100
    end

    if number >= 90 then result = result .. "XC"; number = number - 90
    elseif number >= 80 then result = result .. "LXXX"; number = number - 80
    elseif number >= 70 then result = result .. "LXX"; number = number - 70
    elseif number >= 60 then result = result .. "LX"; number = number - 60
    elseif number >= 50 then result = result .. "L"; number = number - 50
    elseif number >= 40 then result = result .. "XL"; number = number - 40
    elseif number >= 30 then result = result .. "XXX"; number = number - 30
    elseif number >= 20 then result = result .. "XX"; number = number - 20
    elseif number >= 10 then result = result .. "X"; number = number - 10
    end

    if number >= 9 then result = result .. "IX"
    elseif number >= 8 then result = result .. "VIII"
    elseif number >= 7 then result = result .. "VII"
    elseif number >= 6 then result = result .. "VI"
    elseif number >= 5 then result = result .. "V"
    elseif number >= 4 then result = result .. "IV"
    elseif number >= 3 then result = result .. "III"
    elseif number >= 2 then result = result .. "II"
    elseif number >= 1 then result = result .. "I"
    end

    return result;

end

function renderPrices(pos, caption, money, resources)

    drawText(caption, pos.x, pos.y, ColorRGB(1, 1, 1), 15, 0, 0, 2)

    -- render contruction costs
    local py = pos.y + 20
    drawText("$", pos.x, py, ColorRGB(1, 1, 1), 15, 0, 0, 2)
    drawText(createMonetaryString(round(money, 0)), pos.x + 80, py, ColorRGB(1, 1, 1), 15, 0, 0, 2)

    -- render resources costs
    if resources ~= nil then
        py = py + 15
        for i, v in ipairs(resources) do
            local planResources = resources[i]

            if planResources > 0 then
                drawText(Material(i - 1).name, pos.x, py, Material(i - 1).color, 15, 0, 0, 2)
                drawText(createMonetaryString(round(planResources, 0)), pos.x + 80, py, Material(i - 1).color, 15, 0, 0, 2)
                py = py + 15
            end
        end
    end

end

function GetRelationChangeFromMoney(money)
    return money / 210
end

function tablelength(T)
    if T == nil then return 0 end

    local count = 0
    for _ in pairs(T) do count = count + 1 end
    return count
end

function pairsByKeys (t, f)
    local a = {}
    for n in pairs(t) do table.insert(a, n) end
    table.sort(a, f)
    local i = 0      -- iterator variable
    local iter = function ()   -- iterator function
        i = i + 1
        if a[i] == nil then return nil
        else return a[i], t[a[i]]
        end
    end
    return iter
end

-- shamelessly copied from the lua doc
function string:split(sep)
    local sep, fields = sep or ":", {}
    local pattern = string.format("([^%s]+)", sep)
    self:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end
