#include "version.inl"

uniform sampler2D surfaceTexture;

uniform vec3 lightDir;
uniform vec3 lightColor;
uniform vec3 eye;
uniform vec3 atmosphereColor;
uniform vec3 planetPosition;
uniform float planetRadius;

in vec3 position;
in vec2 texCoord;

void main()
{
	vec4 textureColor = texture2D(surfaceTexture, texCoord);

	vec3 x1 = planetPosition;
	vec3 x2 = planetPosition + lightDir;
	vec3 x0 = position;

	vec3 d = abs(cross(x2 - x1, x1 - x0)) / length(x2 - x1);

	float shadow = 1.0;
	float dist = length(d);

	shadow = dist / planetRadius;
	shadow = 1.0 - clamp(1.0 - (shadow - 0.95) / 0.05, 0.0, 1.0);
	shadow = max(0.1, shadow);

	vec3 lightOrigin = planetPosition - lightDir * 1000.0;
	if(length(lightOrigin - x0) < length(lightOrigin - planetPosition))
	{
		shadow = 1.0;
	}

	outFragColor = vec4(textureColor.rgb * shadow, textureColor.a);
}
